// Code generated by util/telemetry/builder. DO NOT EDIT.
package telemetry

const (
	AttribBuildCompiler       string = `compiler`
	AttribBuildDate           string = `build_date`
	AttribBuildGitCommit      string = `git_commit`
	AttribBuildGitTag         string = `git_tag`
	AttribBuildGitTreeState   string = `git_tree_state`
	AttribBuildGoVersion      string = `go_version`
	AttribBuildPlatform       string = `platform`
	AttribBuildVersion        string = `version`
	AttribConcurrencyPolicy   string = `concurrency_policy`
	AttribCronWFName          string = `name`
	AttribCronWFNamespace     string = `namespace`
	AttribDeprecatedFeature   string = `feature`
	AttribErrorCause          string = `cause`
	AttribLogLevel            string = `level`
	AttribNodePhase           string = `node_phase`
	AttribPodNamespace        string = `namespace`
	AttribPodPendingReason    string = `reason`
	AttribPodPhase            string = `phase`
	AttribPodRestartCondition string = `condition`
	AttribPodRestartReason    string = `reason`
	AttribQueueName           string = `queue_name`
	AttribRecentlyStarted     string = `recently_started`
	AttribRequestCode         string = `status_code`
	AttribRequestKind         string = `kind`
	AttribRequestVerb         string = `verb`
	AttribTemplateCluster     string = `cluster_scope`
	AttribTemplateName        string = `name`
	AttribTemplateNamespace   string = `namespace`
	AttribWorkerType          string = `worker_type`
	AttribWorkflowNamespace   string = `namespace`
	AttribWorkflowPhase       string = `phase`
	AttribWorkflowStatus      string = `status`
	AttribWorkflowType        string = `type`
)
